﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SalesVizu
{
    public class Buttons
    {
        public enum Code
        {
            CurrencyDollars,
            CurrencyEuros,
            CurrencyPounds,

            GraphValue,
            GraphSales,
            GraphSalesTrials,

            GamesAll,
            GamesNone,

            TimeAll,
            TimeFirst90Days,
            TimeLast90Days,
            TimeFirst30Days,
            TimeLast30Days,
            TimeFirst7Days,
            TimeLast7Days,
            TimeYear,
            TimeQuarter,
            TimeMonth,
            TimeWeek,
            TimeCustom,

            TimePeriodNext,
            TimePeriodPrevious,
            TimePeriodFirst,
            TimePeriodLast,
            
            // ...
        };

        const int GameCodeStart = 1000;
        public static Code GameCode(int gameIndex)
        {
            return (Code)(gameIndex + GameCodeStart);
        }
        public static int GameIndexFromButtonCode(Code buttonCode)
        {
            int code = (int)buttonCode;
            if (code >= GameCodeStart && code < GameCodeStart + Data.Games.Length)
            {
                return code - GameCodeStart;
            }
            else
            {
                return -1;
            }
        }

        public static List<Button<String>> AllStringButtons = new List<Button<String>>();
        public static List<Button<Texture2D>> AllTextureButtons = new List<Button<Texture2D>>();
        static Button<String> ButtonDollars, ButtonEuro, ButtonPounds;
        public static int ButtonDollarBottom { get { return ButtonDollars.rect.Bottom; } }

        static Texture2D BGon, BGoff;
        public static int Height { get { return BGon.Height; } }
        public static int SourceWidth { get { return BGon.Width; } }
        public enum PressOrRelease { Press, Release };
        public static Texture2D Texture(PressOrRelease State)
        {
            return State == PressOrRelease.Press ? BGon : BGoff;
        }

        public static void Load()
        {
            BGon = MaximinusGame.Instance.Content.Load<Texture2D>("buttonON");
            BGoff = MaximinusGame.Instance.Content.Load<Texture2D>("buttonOFF");

            ButtonDollars = new Button<String>(
                "$",
                new Point(50, 1000),
                PressOrRelease.Release,
                Code.CurrencyDollars);
            ButtonEuro = new Button<String>(
                "€",
                new Point(100, 1000),
                PressOrRelease.Release,
                Code.CurrencyEuros);
            ButtonPounds = new Button<String>(
                "£",
                new Point(150, 1000),
                PressOrRelease.Release,
                Code.CurrencyPounds);
            ShowHideButton(ButtonDollars, false);
            ShowHideButton(ButtonEuro, false);
            ShowHideButton(ButtonPounds, false);

            Button<String> b = new Button<String>(
                "Value",
                new Point(50, 690),
                PressOrRelease.Release,
                Code.GraphValue);
            b.rect = new Rectangle(15, 690, 150, 40);
            b = new Button<String>(
                "Sales",
                new Point(b.rect.Right + 10, 690),
                PressOrRelease.Release,
                Code.GraphSales);
            //b.rect.X += b.rect.Width / 2;
            b.rect = new Rectangle(15, 730, 150, 40);
            b = new Button<String>(
                "Sales & Trials",
                new Point(b.rect.Right + 10, 690),
                PressOrRelease.Release,
                Code.GraphSalesTrials);
            //b.rect.X += b.rect.Width / 2;
            b.rect = new Rectangle(15, 770, 150, 40);
        }


        public static void HandleRightClick(int X, int Y)
        {
            if (LoadSales.Rebuilding || Totals.DataChangeFrame < 0) { return; }
            foreach (Button<String> b in AllStringButtons)
            {
                int gameCodeIndex = GameIndexFromButtonCode(b.code);
                if (gameCodeIndex >= 0 && b.rect.Contains(X, Y))
                {
                    for (int g = 0; g < Data.Games.Length; g++)
                    {
                        Games.Selected[g] = g == gameCodeIndex;
                    }
                    foreach (Button<String> bS in AllStringButtons)
                    {
                        bS.State = ComputeState(bS.code);

                        int gameIndex = GameIndexFromButtonCode(bS.code);
                        if (gameIndex >= 0)
                        {
                            bS.LabelColor = Games.Selected[gameIndex]
                                ? Colors.GameColor(gameIndex)
                                : Color.Transparent;
                        }
                    }
                    LoadSales.Rebuild();
                    return;
                }
            }
        }

        public static void HandleInput(int X, int Y, PressOrRelease pressOrRelease)
        {
            if (LoadSales.Rebuilding || Totals.DataChangeFrame < 0) { return; }

            foreach(Button<String> bS in AllStringButtons)
            {
                bS.HandleInput(X, Y, pressOrRelease);
            }
            foreach (Button<Texture2D> bT in AllTextureButtons)
            {
                bT.HandleInput(X, Y, pressOrRelease);
            }
        }
        public static void DrawAll()
        {
            foreach (Button<String> bS in AllStringButtons)
            {
                bS.DrawBG();
                MaximinusGame.DrawStringOutlined(
                    (String)(object)bS.label,
                    MaximinusGame.Instance.Font,
                    Utils.PointToVec2(bS.rect.Center),
                    bS.LabelColor, 1, 1);
            }
            foreach (Button<Texture2D> bT in AllTextureButtons)
            {
                bT.DrawBG();
                MaximinusGame.Instance.SpriteBatch.Draw(
                    (Texture2D)(object)bT.label,
                    Utils.PointToVec2(bT.rect.Center),
                    null,
                    bT.LabelColor, 0, new Vector2(bT.Width / 2, Buttons.Height / 2), 1, SpriteEffects.None, 0);
            }

            ShowHideButton(ButtonDollars, Currency.DollarsToEUR > 0 || Currency.DollarsToGBP > 0);
            ShowHideButton(ButtonEuro, Currency.DollarsToEUR > 0);
            ShowHideButton(ButtonPounds, Currency.DollarsToGBP > 0);

            //bool canChangeTimePeriod = 
            //    (TimeScale.type == TimeScale.Type.Year || TimeScale.type == TimeScale.Type.Quarter ||
            //        TimeScale.type == TimeScale.Type.Month || TimeScale.type == TimeScale.Type.Week);
            //foreach (Button<Texture2D> changeTimePeriodButton in TimeScale.ChangeTimePeriodPlusButtons)
            //{
            //    ShowHideButton(changeTimePeriodButton, canChangeTimePeriod && TimeScale.AnyDataBeforeAfterPeriod(true));
            //}
            //foreach (Button<Texture2D> changeTimePeriodButton in TimeScale.ChangeTimePeriodMinusButtons)
            //{
            //    ShowHideButton(changeTimePeriodButton, canChangeTimePeriod && TimeScale.AnyDataBeforeAfterPeriod(false));
            //}
        }

        static void ShowHideButton(Button<String> button, bool visible)
        {
            if (visible != button.rect.X > 0)
            {
                button.rect.X *= -1;
            }
        }
        public static void ShowHideButton(Button<Texture2D> button, bool visible)
        {
            if (visible != button.rect.X > 0)
            {
                button.rect.X *= -1;
            }
        }


        public static void Callback(Code code)
        {
            bool? mustRebuild = null;
            int gameCodeIndex = GameIndexFromButtonCode(code);
            if (gameCodeIndex >= 0)
            {
                mustRebuild = true;
                Games.Selected[gameCodeIndex] = !Games.Selected[gameCodeIndex];
            }
            else
            {
                switch (code)
                {
                    case Code.CurrencyDollars:
                        mustRebuild = Currency.type != Currency.Type.Dollars;
                        Currency.type = Currency.Type.Dollars;
                        break;
                    case Code.CurrencyEuros:
                        mustRebuild = Currency.type != Currency.Type.Euros;
                        Currency.type = Currency.Type.Euros;
                        break;
                    case Code.CurrencyPounds:
                        mustRebuild = Currency.type != Currency.Type.Pounds;
                        Currency.type = Currency.Type.Pounds;
                        break;
                    case Code.GraphValue:
                        mustRebuild = BigGraph.type != BigGraph.Type.ValueMSP;
                        BigGraph.type = BigGraph.Type.ValueMSP;
                        break;
                    case Code.GraphSales:
                        mustRebuild = BigGraph.type != BigGraph.Type.Sales;
                        BigGraph.type = BigGraph.Type.Sales;
                        break;
                    case Code.GraphSalesTrials:
                        mustRebuild = BigGraph.type != BigGraph.Type.SalesAndUnsoldTrials;
                        BigGraph.type = BigGraph.Type.SalesAndUnsoldTrials;
                        break;
                    case Code.GamesNone:
                        mustRebuild = false;
                        for (int g = 0; g < Data.Games.Length; g++)
                        {
                            Games.Selected[g] = false;
                        }
                        break;
                    case Code.GamesAll:
                        mustRebuild = false;                        
                        for (int g = 0; g < Data.Games.Length; g++)
                        {
                            if (Games.Selected[g] == false)
                            {
                                mustRebuild = true;
                                Games.Selected[g] = true;
                            }
                        }
                        break;
                    case Code.TimeAll:
                    case Code.TimeFirst90Days:
                    case Code.TimeLast90Days:
                    case Code.TimeFirst30Days:
                    case Code.TimeFirst7Days:
                    case Code.TimeLast30Days:
                    case Code.TimeLast7Days:
                    case Code.TimeMonth:
                    case Code.TimeQuarter:
                    case Code.TimeWeek:
                    case Code.TimeCustom:
                    case Code.TimeYear:
                        mustRebuild = TimeScale.ChangeTo(code);
                        break;
                    case Code.TimePeriodFirst:
                    case Code.TimePeriodLast:
                    case Code.TimePeriodNext:
                    case Code.TimePeriodPrevious:
                        mustRebuild = TimeScale.ChangePeriod(code);
                        break;
                }
            }
            foreach (Button<String> bS in AllStringButtons)
            {
                bS.State = ComputeState(bS.code);

                int gameIndex = GameIndexFromButtonCode(bS.code);
                if (gameIndex >= 0)
                {
                    bS.LabelColor = Games.Selected[gameIndex]
                        ? Colors.GameColor(gameIndex)
                        : Color.Transparent;
                }
            }
            foreach (Button<Texture2D> bT in AllTextureButtons)
            {
                bT.State = ComputeState(bT.code);
            }

            if ((bool)mustRebuild && Games.AnyGameSelected)
            {
                LoadSales.Rebuild();
            }
        }

        public static PressOrRelease ComputeState(Code code)
        {
            int gameIndex = GameIndexFromButtonCode(code);
            if (gameIndex >= 0)
            {
                return Games.Selected[gameIndex] ? PressOrRelease.Press : PressOrRelease.Release;
            }

            switch (code)
            {
                case Code.CurrencyDollars:
                    return Currency.type == Currency.Type.Dollars ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.CurrencyEuros:
                    return Currency.type == Currency.Type.Euros ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.CurrencyPounds:
                    return Currency.type == Currency.Type.Pounds ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.GraphValue:
                    return BigGraph.type == BigGraph.Type.ValueMSP ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.GraphSales:
                    return BigGraph.type == BigGraph.Type.Sales ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.GraphSalesTrials:
                    return BigGraph.type == BigGraph.Type.SalesAndUnsoldTrials ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.GamesAll:
                case Code.GamesNone:
                    return PressOrRelease.Press;

                case Code.TimeAll:
                case Code.TimeFirst90Days:
                case Code.TimeLast90Days:
                case Code.TimeFirst30Days:
                case Code.TimeFirst7Days:
                case Code.TimeLast30Days:
                case Code.TimeLast7Days:
                case Code.TimeMonth:
                case Code.TimeQuarter:
                case Code.TimeWeek:
                case Code.TimeCustom:
                case Code.TimeYear:
                    return TimeScale.type == TimeScale.ButtonCodeToType(code) ? PressOrRelease.Press : PressOrRelease.Release;
                case Code.TimePeriodFirst:
                case Code.TimePeriodLast:
                case Code.TimePeriodNext:
                case Code.TimePeriodPrevious:
                    return PressOrRelease.Press;
            }
            throw new NotImplementedException();
        }
    }

    public class Button<T>
    {        

        public Rectangle rect;
        public Buttons.PressOrRelease State;
        public readonly T label;
        public readonly Buttons.Code code;
        readonly Buttons.PressOrRelease pressOrRelease;
        public Color LabelColor = Color.White;

        public int Width
        {
            get
            {
                if (typeof(T) == typeof(String))
                {
                    return (int)MaximinusGame.Instance.Font.MeasureString((String)(object)label).X + 20;
                }
                else if (typeof(T) == typeof(Texture2D))
                {
                    return ((Texture2D)(object)label).Width;
                }
                throw new NotImplementedException();
            }
        }

        public Button(T label, Point centerPos, Buttons.PressOrRelease pressOrRelease, Buttons.Code code)
        {
            this.label = label;
            this.rect = new Rectangle(centerPos.X - Width / 2, centerPos.Y - Buttons.Height / 2, Width, Buttons.Height);
            this.pressOrRelease = pressOrRelease;
            this.code = code;
            if (typeof(T) == typeof(String))
            {
                Buttons.AllStringButtons.Add((Button<String>)(object)this);
            }
            else if (typeof(T) == typeof(Texture2D))
            {
                Buttons.AllTextureButtons.Add((Button<Texture2D>)(object)this);
            }
            else { throw new NotImplementedException(); }
            State = Buttons.ComputeState(code);
        }

        public void HandleInput(int X, int Y, Buttons.PressOrRelease pressOrRelease)
        {
            if (rect.X >= 0 && rect.Contains(X, Y) && pressOrRelease == this.pressOrRelease)
            {
                Buttons.Callback(code);
            }
        }
                

        public void DrawBG()
        {
            Texture2D tex = Buttons.Texture(State);
            const int columnWidth = 2;
            MaximinusGame.Instance.SpriteBatch.Draw(tex,
                new Rectangle(
                    rect.X, rect.Y,
                    columnWidth, rect.Height),
                new Rectangle(
                    0, 0,
                    columnWidth, Buttons.Height),
                Color.White);
            MaximinusGame.Instance.SpriteBatch.Draw(tex,
                new Rectangle(
                    rect.X + columnWidth, rect.Y,
                    rect.Width - 2 * columnWidth, rect.Height),
                new Rectangle(
                    columnWidth, 0,
                    Buttons.SourceWidth - 2 * columnWidth, Buttons.Height),
                Color.White);
            MaximinusGame.Instance.SpriteBatch.Draw(tex,
                new Rectangle(
                    rect.Right - columnWidth, rect.Y,
                    columnWidth, Buttons.Height),
                new Rectangle(
                    Buttons.SourceWidth - columnWidth, 0,
                    columnWidth, Buttons.Height),
                Color.White);


        }
    }
}
