using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.IO;

namespace SalesVizu
{
    public class Game1 : MaximinusGame
    {

        public static RenderTarget2D RenderMain;

        const float aspect =
            //16f/9, 
            16f / 8.85f;

        public Game1()
            : base(
                Settings.WidthPixels,
                aspect)
                //(MaximinusGame.ID)(-1),
                ////1920,
                //Settings.WidthPixels,
                //"Sales Vizu",
                //LoadSales.LoadCB,
                //aspect,
                //true,
                //false,
                //false,
                //false,
                //false,
                //false,
                //false,
                //new Camera3D.Setup(false))
        {
            if (Settings.Success == false)
            {
                String tmpdir = System.Environment.GetEnvironmentVariable("TMP");
                String errorfile = tmpdir + "\\crashlog.txt";
                StreamWriter sw = new StreamWriter(errorfile);
                sw.WriteLine("Error parsing settings file");
                sw.WriteLine(Settings.userDir == null ? "Error" : Settings.userDir + "\\" + Settings.fileName.Replace("\\", ""));
                //sw.WriteLine("in directory " + Settings.userDir == null ? "Error" : Settings.userDir);
                sw.WriteLine("");
                sw.WriteLine(Settings.ParseError);
                sw.WriteLine(Utils.newLine + "Settings file must look like this ->" + Utils.newLine + Utils.newLine +
                    "Width=1900" + Utils.newLine +
                    "DataDir=Z:\\Devel\\Data\\Xna_Sales" + Utils.newLine +
                    "IncomingDir=Z:\\Incoming" + Utils.newLine +
                    "Wget=c:\\bin\\wget" + Utils.newLine +
                    Utils.newLine +
                    "Width : mandatory, specify the window width in pixels, must be less than 1900" + Utils.newLine +
                    "DataDir : mandatory, the directory where your sales files are" + Utils.newLine +
                    Utils.newLine +
                    "IncomingDir and Wget are optional" + Utils.newLine +
                    Utils.newLine +
                    "This program will move sales files" + Utils.newLine +
                    "from IncomingDir to DataDir and use wget to fetch exchange rates");
                //sw.Write("Error parsing settings file " + Settings.fileName.Replace("\\", "") + " in directory " + Settings.userDir == null ? "Error" : Settings.userDir);
                sw.Close();
                MaximinusGame.LaunchSystemCommand("notepad", errorfile, false, true);
                Exit();
            }


            Content.RootDirectory = "Content";
            IsMouseVisible = true;
        
        }

        protected override void Initialize()
        {
            base.Initialize();
            RenderMain = LibMaximinus.NewRenderTarget(1900, (int)((float)1900 / aspect));
        }

        Threading.ManagedThread threadLoadContent;

        protected override void LoadContent()
        {
            threadLoadContent = new Threading.ManagedThread();
            threadLoadContent.AddTask(new Threading.ThreadTask(LoadSales.LoadCB));

            base.LoadContent();
        }


        public static PieChart ChartByGame, ChartByCountry;
        public static void Load()
        {
            ChartByGame = new PieChart("By Game", Totals.ValueMSPbyGame, ByGame.ColorOf, ByGame.LabelOf, 250);
            ChartByCountry = new PieChart("By Country", Totals.ValueMSPbyCountry, ByCountry.ColorOf, ByCountry.LabelOf, ChartByGame.Right + 20);
        }


        //protected override void Initialize() { base.Initialize(); }
        //protected override void LoadContent() { }


        MouseState mousePrev;

        protected override void OnExiting(object sender, EventArgs args)
        {
            LoadSales.computeThread.KillImmediately();
            LoadSales.fetchRatesThread.KillImmediately();

            base.OnExiting(sender, args);
        }

        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyDown(Keys.Escape)) { Exit(); }


            if (threadLoadContent != null && MaximinusGame.ContentLoadFinished)
            {
                threadLoadContent.Kill();
                threadLoadContent = null;
            }

            MouseState mouseCurr = Mouse.GetState();
            if (mousePrev != null && IsActive)
            {
                Point mousePos = new Point(
                    mouseCurr.X * 1900 / MaximinusGame.Instance.ScreenSizePoint.X,
                    mouseCurr.Y * 1900 / MaximinusGame.Instance.ScreenSizePoint.X);


                if (mouseCurr.LeftButton == ButtonState.Released && mousePrev.LeftButton == ButtonState.Pressed)
                {
                    Buttons.HandleInput(mousePos.X, mousePos.Y, Buttons.PressOrRelease.Release);
                }
                else if (mouseCurr.LeftButton == ButtonState.Pressed)
                {
                    Buttons.HandleInput(mousePos.X, mousePos.Y, Buttons.PressOrRelease.Press);
                }
                if (mouseCurr.RightButton == ButtonState.Released && mousePrev.RightButton == ButtonState.Pressed)
                {
                    Buttons.HandleRightClick(mousePos.X, mousePos.Y);
                }
                mousePrev = mouseCurr;
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            //base.Draw(gameTime);


            if (MaximinusGame.ContentLoadFinished == false)
            {
                MaximinusGame.Instance.GraphicsDevice.Clear(new Color(Vector3.One * .90f));
                return;
            }

            bool anyGameSelected = Games.AnyGameSelected;
            if (anyGameSelected)
            {
                ChartByGame.PreDraw();
                ChartByCountry.PreDraw();
                ConvRates.PreDraw();
                BigGraph.PreDraw();
            }
            MaximinusGame.Instance.GraphicsDevice.SetRenderTarget(RenderMain);
            MaximinusGame.Instance.GraphicsDevice.Clear(new Color(Vector3.One * .90f));

            MaximinusGame.Instance.SpriteBatch.Begin();

            //Games.Draw();
            Totals.Draw(anyGameSelected);
            if (anyGameSelected && LoadSales.Rebuilding == false && BigGraph.DrawOnFrame >= 0)
            {
                ChartByGame.Draw();
                ChartByCountry.Draw();
                ConvRates.Draw();
                BigGraph.Draw();
                TimeScale.Draw();
            }
            Buttons.DrawAll();
            Currency.Draw();

            MaximinusGame.Instance.SpriteBatch.End();
            MaximinusGame.Instance.GraphicsDevice.SetRenderTarget(null);
            MaximinusGame.Instance.SpriteBatch.Begin();
            MaximinusGame.Instance.SpriteBatch.Draw(RenderMain, MaximinusGame.Instance.GraphicsDevice.Viewport.Bounds, Color.White);
            MaximinusGame.Instance.SpriteBatch.End();
        }
    }
}
