﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;

namespace SalesVizu
{
    public class GameWithDate
    {
        public String GameName;
        public int Year;
        public int Month;
        public int Day;
    }

    public class LoadSales
    {

        static AutoResetEvent computeEvent;
        public static Threading.ManagedThread computeThread, fetchRatesThread;



        public static void LoadCB()
        {

            fetchRatesThread = new Threading.ManagedThread();
            fetchRatesThread.AddTask(new Threading.ThreadTask(Currency.FetchRates));

            computeEvent = new AutoResetEvent(false);
            computeThread = new Threading.ManagedThread();
            computeThread.AddTask(new Threading.ThreadTask(computeCallback));

            while (Settings.Success == false) { }

            MoveNewFiles();

            Buttons.Load();

            BuildGameList();

            PieChart.Load();

            Game1.Load();

            ConvRates.Load();

            BigGraph.Load();

            TimeScale.Load();

            MaximinusGame.ContentLoadFinished = true;

            ReadSalesData();

            Rebuild();
        }

        public static void Rebuild() 
        { 
            Utils.DebugOut("REBUILD start");
            Rebuilding = true;
            computeEvent.Set(); 
        }
        public static bool Rebuilding = false;

        static void computeCallback()
        {
            while (true)
            {
                computeEvent.WaitOne();
                Totals.Rebuild();
                Utils.DebugOut("REBUILD end");
                Rebuilding = false;
            }
        }

        static void MoveNewFiles()
        {
            try
            {
                foreach (String fileName in Directory.EnumerateFiles(Settings.IncomingDir, pattern).ToList())
                {
                    String[] fields = fileName.Split('\\');
                    String dest = Settings.DataDir + "\\" + fields[fields.Length - 1];

                    File.Copy(fileName, dest, true);
                    File.Delete(fileName);
                }
            }
            catch (Exception e) 
            {
                Utils.DebugOut(e);
            }
        }

        public static readonly String pattern = "*_*_????.csv";

        static void BuildGameList()
        {
            List<GameWithDate> earliestFiles = new List<GameWithDate>();
            List<string> files = Directory.EnumerateFiles(Settings.DataDir, pattern).ToList();
            foreach(String fileName in files)
            {
                CheckIfEarliest(ParseFileName(fileName), earliestFiles);
            }

            List<GameWithDate> gamesSortedByDate = new List<GameWithDate>();
            int maxLength = 0;
            foreach (GameWithDate game in earliestFiles)
            {
                int insertIndex = 0;
                for (int i = 0; i < gamesSortedByDate.Count; i++)
                {
                    if (earliestFiles[i].Year < game.Year 
                        || (earliestFiles[i].Year == game.Year && earliestFiles[i].Month < game.Month)
                        || (earliestFiles[i].Year == game.Year && earliestFiles[i].Month == game.Month && earliestFiles[i].Day < game.Day))
                    {
                        insertIndex++;
                    }
                }
                gamesSortedByDate.Insert(insertIndex, game);
                maxLength = Math.Max(maxLength, game.GameName.Length);
            }
            Data.Games = gamesSortedByDate.ToArray();
            Data.MaxGameNameLength = maxLength;
            Games.Load();

        }

        static GameWithDate ParseFileName(String fileName)
        {
            String[] basenameFields = fileName.Split('\\');


            String[] fields = basenameFields[basenameFields.Length - 1].Split('_');
            GameWithDate ret = new GameWithDate();
            ret.GameName = fields[0];
            ret.Month = int.Parse(fields[1]);
            ret.Year = int.Parse(fields[2].Substring(0, 4));

            StreamReader s = new StreamReader(fileName);
            String all = s.ReadToEnd();
            String[] lines = all.Split('\n');
            String lastLine = lines[lines.Length - 2];
            String[] lastLineFields = lastLine.Split(',');
            lastLineFields = lastLineFields[0].Split('/');
            ret.Day = int.Parse(lastLineFields[1]);
            s.Close();

            return ret;
        }

        static void CheckIfEarliest(GameWithDate game, List<GameWithDate> earliestFiles)
        {
            foreach (GameWithDate gameInList in earliestFiles)
            {
                if (game.GameName == gameInList.GameName)
                {
                    if (game.Year < gameInList.Year 
                        || (game.Year == gameInList.Year && game.Month < gameInList.Month)
                        || (game.Year == gameInList.Year && game.Month == gameInList.Month && game.Day < gameInList.Day))
                    {
                        gameInList.Year = game.Year;
                        gameInList.Month = game.Month;
                        gameInList.Day = game.Day;
                    }
                    return;
                }
            }
            earliestFiles.Add(game);
        }

        static void ReadSalesData()
        {
            Data.SalesByGame = new List<DailySales>[Data.Games.Length];

            List<string> files = Directory.EnumerateFiles(Settings.DataDir, pattern).ToList();
            foreach (String fileName in files)
            {
                GameWithDate game = ParseFileName(fileName);
                int gameIndex = FindGameIndex(game.GameName);
                if (Data.SalesByGame[gameIndex] == null)
                {
                    Data.SalesByGame[gameIndex] = new List<DailySales>();
                }

                StreamReader s = new StreamReader(fileName);
                s.ReadLine();
                while (s.EndOfStream == false)
                {
                    String line = s.ReadLine();
                    String[] fields = line.Split(',');
                    String dateString = fields[0].Replace('\"', ' ');
                    InsertSales(
                        Data.SalesByGame[gameIndex],                       // list of daily sales
                        dateString,                                        // date
                        int.Parse(fields[2].Replace('\"', ' ')),           // value
                        int.Parse(fields[3].Replace('\"', ' ')),           // count
                        fields[1].Replace('\"', ' ').Trim()                // country
                    );
                    InsertDate(dateString, gameIndex);
                }
                s.Close();
            }

            // Compute WeekMonthQuarterYear_ValuesOfDates for each date
            WeekMonthQuarterYear_ValuesOfDates = new int[AllDates.Count][];
            WeekMonthQuarterYear_ValuesOfDates[0] = new int[] { 0, 0, 0, 0 };
            for (int i = 1; i < AllDates.Count; i++)
            {
                WeekMonthQuarterYear_ValuesOfDates[i] = new int[] { -1, -1, -1, -1 };

                int
                    previousDayOfWeek = (int)new DateTime(AllDates[i-1].Year, AllDates[i-1].Month, AllDates[i-1].Day).DayOfWeek,
                    currentDayOfWeek = (int)new DateTime(AllDates[i].Year, AllDates[i].Month, AllDates[i].Day).DayOfWeek;
                previousDayOfWeek = (previousDayOfWeek + 6) % 7;
                currentDayOfWeek = (currentDayOfWeek + 6) % 7;
                
                WeekMonthQuarterYear_ValuesOfDates[i][0] =
                    currentDayOfWeek > previousDayOfWeek
                    ? WeekMonthQuarterYear_ValuesOfDates[i - 1][0]
                    : (WeekMonthQuarterYear_ValuesOfDates[i - 1][0] + 1);

                WeekMonthQuarterYear_ValuesOfDates[i][1] =
                    AllDates[i - 1].Month == AllDates[i].Month
                    ? WeekMonthQuarterYear_ValuesOfDates[i - 1][1]
                    : (WeekMonthQuarterYear_ValuesOfDates[i - 1][1] + 1);

                WeekMonthQuarterYear_ValuesOfDates[i][2] =
                    (AllDates[i - 1].Month-1) / 3 == (AllDates[i].Month-1) / 3
                    ? WeekMonthQuarterYear_ValuesOfDates[i - 1][2]
                    : (WeekMonthQuarterYear_ValuesOfDates[i - 1][2] + 1);

                WeekMonthQuarterYear_ValuesOfDates[i][3] =
                    AllDates[i - 1].FiscalYear == AllDates[i].FiscalYear
                    ? WeekMonthQuarterYear_ValuesOfDates[i - 1][3]
                    : (WeekMonthQuarterYear_ValuesOfDates[i - 1][3] + 1);
            }
        }

        public static int[][] WeekMonthQuarterYear_ValuesOfDates;

        public struct DateWithGames
        {
            public readonly int Year, Month, Day;
            public List<int> GamesWithAnyActivity;
            public int FiscalYear
            {
                get
                {
                    return
                        this.Year
                        + (this.Month > 9
                        ? 1 : 0);
                }
            }
            public DateWithGames(int Y, int M, int D)
            {
                this.Year = Y;
                this.Month = M;
                this.Day = D;
                this.GamesWithAnyActivity = new List<int>();
            }
            public override bool  Equals(object obj)
            {
                DateWithGames other = (DateWithGames)obj;
                return
                    this.Year == other.Year
                    && this.Month == other.Month
                    && this.Day == other.Day;
            }
            public override int GetHashCode() { throw new NotImplementedException(); }

            public bool IsEarlierThan(DateWithGames other)
            {
                return
                    this.Year < other.Year
                    || (this.Year == other.Year && this.Month < other.Month)
                    || (this.Year == other.Year && this.Month == other.Month && this.Day < other.Day);
            }
            public bool ContainsAnySelectedGame
            {
                get
                {
                    foreach (int gameIndex in this.GamesWithAnyActivity)
                    {
                        if (Games.Selected[gameIndex]) { return true; }
                    }
                    return false;
                }
            }
            public override string ToString()
            {
                return
                    Day.ToString("0") + DaySuffix(Day) +
                    MonthString(Month-1) + " " +
                    Year.ToString("0");
            }
            static String DaySuffix(int day)
            {
                if (day % 10 == 1 && day != 11) { return "st "; }
                if (day % 10 == 2 && day != 12) { return "nd "; }
                if (day % 10 == 3 && day != 13) { return "rd "; }
                return "th ";
            }
            public static String MonthString(int month)
            {
                return
                    month == 0 ? "January" :
                    month == 1 ? "February" :
                    month == 2 ? "March" :
                    month == 3 ? "April" :
                    month == 4 ? "May" :
                    month == 5 ? "June" :
                    month == 6 ? "July" :
                    month == 7 ? "August" :
                    month == 8 ? "September" :
                    month == 9 ? "October" :
                    month == 10 ? "November" :
                    month == 11 ? "December" : "";
            }
        }
        public static List<DateWithGames> AllDates = new List<DateWithGames>();

        static void InsertDate(String dateString, int gameIndex)
        {
            String[] dateFields = dateString.Split('/');
            DateWithGames d = new DateWithGames(
                int.Parse(dateFields[2]),
                int.Parse(dateFields[0]),            
                int.Parse(dateFields[1]));
            d.GamesWithAnyActivity.Add(gameIndex);
            for (int i = 0; i < AllDates.Count; i++)
            {
                if (AllDates[i].Equals(d))
                {
                    if (AllDates[i].GamesWithAnyActivity.Contains(gameIndex) == false)
                    {
                        AllDates[i].GamesWithAnyActivity.Add(gameIndex);
                    }
                    return;
                }
                else if(d.IsEarlierThan(AllDates[i]))
                {
                    AllDates.Insert(i, d);
                    return;
                }
            }
            AllDates.Add(d);
        }

        static void InsertSales(
            List<DailySales> dailySalesForThisGame,
            String date,
            int value,
            int count,
            String countryName)
        {
            String[] dateFields = date.Split('/');
            DailySales d = new DailySales();
            d.Year = int.Parse(dateFields[2]);
            d.Day = int.Parse(dateFields[1]);
            d.Month = int.Parse(dateFields[0]);

            d.Trials = 0;
            d.Sales = 0;
            d.ValueMSP = 0;

            if (value > 0)
            {
                d.Sales = count;
                d.ValueMSP = count * value;
            }
            else
            {
                d.Trials = count;
            }

            int dayIndex = FindDailySaleIndex(dailySalesForThisGame, d.Year, d.Month, d.Day);

            DailySales currentRecordForThisDay =
                dayIndex <= dailySalesForThisGame.Count - 1
                ? dailySalesForThisGame[dayIndex]
                : null;
            int countryIndex = ByCountry.CountryIndex(countryName);

            if (   currentRecordForThisDay != null
                && currentRecordForThisDay.Year == d.Year
                && currentRecordForThisDay.Month == d.Month
                && currentRecordForThisDay.Day == d.Day)
            {
                // there's already a record for current Day/Game combination

#warning support REPORT BUG "sales are reported as ZERO MSP value"
                int
                    currentRecordTrials = currentRecordForThisDay.TrialsByCountry[countryIndex],
                    newRecordTrials = d.Trials;
                if (currentRecordTrials > 0 && newRecordTrials > 0)
                {
                    //if ((d.Year > 2013 ||
                    //    (d.Year == 2013 && d.Month > 7) ||
                    //    (d.Year == 2013 && d.Month == 7 && d.Day > 26)) == false)
                    //{
                    //    if (System.Diagnostics.Debugger.IsAttached)
                    //    {
                    //        // should not be able to find the report bug before 26 August 2013
                    //        YES it's possible : when redeeming a code, 2 zero-value lines appear
                    //        System.Diagnostics.Debugger.Break();
                    //    }
                    //}


                    // BUG detected : this is another record for same day / same country / and both have trials and no sales
                    // FIX = change the item with lowest count in trials from CURRENT VALUE to ZERO
                    // ... then change the other item in value from ZERO to CURRENT VALUE
                    bool isRecordToChangeCurrent = currentRecordTrials < newRecordTrials;
                    int countOfTrialsWhichAreActuallySales = isRecordToChangeCurrent ? currentRecordTrials : newRecordTrials;
#warning assuming all sales are 80 MSP (but some can be 240 or 500)
                    int valueOfTrialsWhichAreActuallySales = countOfTrialsWhichAreActuallySales * 80;
                    DailySales itemToChange = // <-> item whose trials value actually are SALES
                        isRecordToChangeCurrent
                        ? currentRecordForThisDay 
                        : d;

                    itemToChange.Sales += countOfTrialsWhichAreActuallySales;
                    itemToChange.ValueMSP += valueOfTrialsWhichAreActuallySales;
                    if (isRecordToChangeCurrent)
                    {
                        itemToChange.ValueMSPbyCountry[countryIndex] += valueOfTrialsWhichAreActuallySales;
                    }

                    itemToChange.Trials -= countOfTrialsWhichAreActuallySales;
                    if (isRecordToChangeCurrent)
                    {
                        itemToChange.TrialsByCountry[countryIndex] -= countOfTrialsWhichAreActuallySales;
                    }
                }

                currentRecordForThisDay.Sales += d.Sales;
                currentRecordForThisDay.Trials += d.Trials;
                currentRecordForThisDay.ValueMSP += d.ValueMSP;

                currentRecordForThisDay.TrialsByCountry[countryIndex] += d.Trials;
                currentRecordForThisDay.ValueMSPbyCountry[countryIndex] += d.ValueMSP;
            }
            else
            {
                d.ValueMSPbyCountry = new int[ByCountry.NbCountries];
                d.ValueMSPbyCountry[countryIndex] = d.ValueMSP;

                d.TrialsByCountry = new int[ByCountry.NbCountries];
                d.TrialsByCountry[countryIndex] = d.Trials;

                dailySalesForThisGame.Insert(dayIndex, d);
            }
        }

        static int FindDailySaleIndex(List<DailySales> dailySales, int year, int month, int day)
        {
            for(int i=0; i<dailySales.Count; i++)
            {
                DailySales item = dailySales[i];
                if (
                    (item.Year == year && item.Month == month && item.Day == day)
                    || item.Year > year
                    || (item.Year == year && item.Month > month)
                    || (item.Year == year && item.Month == month && item.Day > day))
                {
                    return i;
                }
            }
            return dailySales.Count;
        }

        static int FindGameIndex(String gameName)
        {
            for (int g = 0; g < Data.Games.Length; g++)
            {
                if (Data.Games[g].GameName == gameName) { return g; }
            }
            throw new Exception();
        }
    }
}
