﻿
//#define DEBUG_BG

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SalesVizu
{
    public class PieChart
    {
        static Texture2D Circle, Radius;
        public static void Load()
        {
            Circle = MaximinusGame.Instance.Content.Load<Texture2D>("Circle");
            Radius = MaximinusGame.Instance.Content.Load<Texture2D>("CircleRadius");
        }

        public delegate Color ColorsCallback(float f);
        public delegate String LabelCallback(int i);

        RenderTarget2D render;
        readonly Rectangle DestRect;
        readonly String label;
        readonly Vector2 origin;
        readonly ColorsCallback colorCB;
        readonly LabelCallback labelCB;
        readonly List<float> data;
        //readonly int columns;

        public int Right { get { return DestRect.Right; } }
        public int Top { get { return DestRect.Top; } }

        public PieChart(String label, List<float> data, ColorsCallback colorCB, LabelCallback labelCB, int startX)
        {
            int Height = (Game1.RenderMain.Width - 350) * 2 / 9;
            int Width = Height * (3 * 9) / (2 * 8);
            if (label.ToLower().Contains("country")) { Width *= 9; Width /= 10; }
            render = LibMaximinus.NewRenderTarget(Width, Height);
            DestRect = new Rectangle(startX, Game1.RenderMain.Height - Height, Width, Height);
            origin = new Vector2(Height / 2, Height / 2);

            this.label = label;
            this.data = data;
            this.colorCB = colorCB;
            this.labelCB = labelCB;
        }

        public int DrawOnFrame = -1;

        public void PreDraw()
        {

            if (LoadSales.Rebuilding == false && Totals.DataChangeFrame >= 0 && DrawOnFrame < Totals.DataChangeFrame)
            {
                DrawOnFrame = MaximinusGame.CurrentFrame;

                MaximinusGame.Instance.GraphicsDevice.SetRenderTarget(render);
                MaximinusGame.Instance.GraphicsDevice.Clear(Color.Transparent);
    #if DEBUG_BG
                MaximinusGame.Draw2D.Device.Clear(label.ToLower().Contains("game") ? Color.Red : Color.Blue);
    #endif
                MaximinusGame.Instance.SpriteBatch.Begin();

                MaximinusGame.DrawStringOutlined(
                    label,
                    MaximinusGame.Instance.Font,
                    new Vector2(DestRect.Width * (5 - (label.ToLower().Contains("country") ? .1f : .2f)) / 6, DestRect.Height * .1f),
                    Color.White, 1, 1);

                float ratio = 0;
                while (ratio < 1)
                {
                    MaximinusGame.Instance.SpriteBatch.Draw(Radius,
                        origin,
                        null,
                        colorCB(ratio),
                        MathHelper.TwoPi * -ratio,
                        origin,
                        1, SpriteEffects.None, 0);

                    ratio += 1f / (MathHelper.Pi * DestRect.Height * 1.7f);
                }


                float gapRatio = 0;
                for(int g = 1; g < data.Count; g++)
                {
                    MaximinusGame.Instance.SpriteBatch.Draw(Radius,
                        origin,
                        null,
                        Color.Black,
                        MathHelper.TwoPi * -gapRatio,
                        origin,
                        1, SpriteEffects.None, 0);
                    gapRatio += data[g - 1];
                }

                for(int g = 0; g < data.Count; g++)
                {
                    new Vector2(DestRect.Width * 5/6, DestRect.Height * (.2f + g * .8f / (data.Count)));


                    if(label.ToLower().Contains("game"))
                    {
                        MaximinusGame.DrawStringOutlinedNotCentered(
                            labelCB(g),
                            //Utils.PointToVec2(Utils.Vec2ToPoint(
                            new Vector2(DestRect.Width * (8.25f + (int)(g / 10) * 2.5f - 1) / 12, DestRect.Height * (.2f + .08f * ((g % 10)-.5f))),
                            Colors.GameColor(g), 1, 1);
                    }
                    else if (label.ToLower().Contains("country"))
                    {
                        String[] fields = labelCB(g).Split(' ');
                        MaximinusGame.DrawStringOutlinedNotCentered(
                            fields[0],
                            //Utils.PointToVec2(Utils.Vec2ToPoint(
                            new Vector2(DestRect.Width * (9.0f - 1) / 12, DestRect.Height * (.2f + (.8f / data.Count) * (g - .5f))),
                            Colors.GameColor(g), 1, 1);
                        MaximinusGame.DrawStringOutlinedNotCentered(
                            fields[1],
                            //Utils.PointToVec2(Utils.Vec2ToPoint(
                            new Vector2(DestRect.Width * (11.3f - 1) / 12, DestRect.Height * (.2f + (.8f / data.Count) * (g - .5f))),
                            Colors.GameColor(g), 1, 1);
                    }
                }
                MaximinusGame.Instance.SpriteBatch.Draw(Circle, Vector2.Zero, Color.White);

                MaximinusGame.Instance.SpriteBatch.End();
            }
        }

        public void Draw()
        {
            MaximinusGame.Instance.SpriteBatch.Draw(render, DestRect, Color.White);
        }
    }
}
