﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Xna.Framework;

namespace SalesVizu
{
    public class Settings
    {
        public static readonly String fileName = "\\SalesVizuSettings.txt";
        public static String userDir = null, ParseError = "";
        static int widthPixels = -1;
        public static int WidthPixels
        {
            get
            {
                if (widthPixels < 0)
                {
                    Load();
                }
                return widthPixels;
            }
        }

        static void Load()
        {
            userDir = 
                System.Environment.GetEnvironmentVariable("APPDATA");
                //MaximinusGame.GetEnvironmentVariable("APPDATA");

            //userDir = userDir.Replace("\r", "");
            //userDir = userDir.Replace("\n", "");
            try
            {
                if (File.Exists(userDir + fileName) == false)
                {
                    ParseError += "File not found";
                    throw new Exception();
                }


                StreamReader s = new StreamReader(userDir + fileName);
                while (!s.EndOfStream)
                {
                    String[] fields = s.ReadLine().Split('=');
                    if (fields[0] == "Width")
                    {
                        if (int.TryParse(fields[1], out widthPixels) == false)
                        {
                            ParseError += "Could not parse Width=INTEGER" + Utils.newLine;
                            throw new Exception();
                        }
                        if (widthPixels > 1900)
                        {
                            ParseError += "Width must be <= 1900" + Utils.newLine;
                            throw new Exception();
                        }
                    }
                    else if (fields[0] == "Wget")
                    {
                        WgetBinary = fields[1];
                    }
                    else if (fields[0] == "DataDir")
                    {
                        DataDir = fields[1];
                        if (Directory.Exists(DataDir) == false)
                        {
                            ParseError += "DataDir " + DataDir + " does not exist";
                            throw new Exception();
                        }


                        int nbFiles = Directory.EnumerateFiles(Settings.DataDir, LoadSales.pattern).ToArray().Length;
                        if (nbFiles == 0)
                        {
                            ParseError += "no sales files matching " + LoadSales.pattern + " in " + Settings.DataDir;
                            throw new Exception();
                        }


                    }
                    else if (fields[0] == "IncomingDir")
                    {
                        IncomingDir = fields[1];
                        if (Directory.Exists(IncomingDir) == false)
                        {
                            ParseError += "IncomingDir " + IncomingDir + " does not exist";
                            throw new Exception();
                        }
                    }
                }
                s.Close();

                if (widthPixels < 0)
                {
                    widthPixels = 1900;
                    ParseError += "Did not find Width=INTEGER" + Utils.newLine;
                    throw new Exception();
                }
                if (DataDir == null)
                {
                    ParseError += "Did not find DataDir=PathToDirectory" + Utils.newLine;
                    throw new Exception();
                }

                Success = true;
            }
            catch (Exception) 
            {
                widthPixels = 1900;
            }
         }

        public static String
            WgetBinary = "",
            DataDir = null,
            IncomingDir = null;
        public static bool Success = false;


    }
}
