﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SalesVizu
{
    public class TimeScale
    {
        public enum Type { All, First90D, Last90D, First30D, Last30D, First7D, Last7D, Year, Quarter, Month, Week, Custom };
        public static Type type = Type.All;
        public static List<Button<Texture2D>> 
            ChangeTimePeriodPlusButtons = new List<Button<Texture2D>>(),
            ChangeTimePeriodMinusButtons = new List<Button<Texture2D>>();

        public static void Load()
        {
            int widthLabel = 120;
            Rectangle buttonRect = new Rectangle(15, BigGraph.Bottom + 05, widthLabel, 40);
            Button<String> b;

            b = new Button<String>(
                "All",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeAll);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;

            buttonRect.Y -= Buttons.Height;
            buttonRect.X += buttonRect.Width + 20;
            b = new Button<String>(
                "First 90 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeFirst90Days);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;
            b = new Button<String>(
                "Last 90 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeLast90Days);
            b.rect = buttonRect;

            buttonRect.Y -= Buttons.Height;
            buttonRect.X += buttonRect.Width;// +20;
            b = new Button<String>(
                "First 30 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeFirst30Days);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;
            b = new Button<String>(
                "Last 30 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeLast30Days);
            b.rect = buttonRect;

            buttonRect.Y -= Buttons.Height;
            buttonRect.X += buttonRect.Width;
            b = new Button<String>(
                "First 7 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeFirst7Days);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;
            b = new Button<String>(
                "Last 7 Days",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeLast7Days);
            b.rect = buttonRect;

            buttonRect.X += buttonRect.Width + 20;
            buttonRect.Y -= Buttons.Height;
            b = new Button<String>(
                "Year",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeYear);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;
            b = new Button<String>(
                "Quarter",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeQuarter);
            b.rect = buttonRect;
            buttonRect.Y -= Buttons.Height;
            buttonRect.X += buttonRect.Width;
            b = new Button<String>(
                "Month",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeMonth);
            b.rect = buttonRect;
            buttonRect.Y += Buttons.Height;
            b = new Button<String>(
                "Week",
                buttonRect.Center,
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimeWeek);
            b.rect = buttonRect;


            int changePeriodXCenter = BigGraph.Left + BigGraph.Width * 3 / 4;
            ChangeTimePeriodMinusButtons.Add(new Button<Texture2D>(
                MaximinusGame.Instance.Content.Load<Texture2D>("buttonFirst"),
                new Point(changePeriodXCenter - 150, buttonRect.Center.Y),
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimePeriodFirst));
            ChangeTimePeriodMinusButtons.Add(new Button<Texture2D>(
                MaximinusGame.Instance.Content.Load<Texture2D>("buttonPrevious"),
                new Point(changePeriodXCenter - 110, buttonRect.Center.Y),
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimePeriodPrevious));
            buttonRect.X += buttonRect.Width + 20;
            ChangeTimePeriodPlusButtons.Add(new Button<Texture2D>(
                MaximinusGame.Instance.Content.Load<Texture2D>("buttonNext"),
                new Point(changePeriodXCenter + 110, buttonRect.Center.Y),
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimePeriodNext));
            ChangeTimePeriodPlusButtons.Add(new Button<Texture2D>(
                MaximinusGame.Instance.Content.Load<Texture2D>("buttonLast"),
                new Point(changePeriodXCenter + 150, buttonRect.Center.Y),
                Buttons.PressOrRelease.Release,
                Buttons.Code.TimePeriodLast));

            HideChangePeriodButtons();

        }

        public static Type ButtonCodeToType(Buttons.Code code)
        {
            switch (code)
            {
                case Buttons.Code.TimeAll:
                    return TimeScale.Type.All;
                case Buttons.Code.TimeFirst90Days:
                    return TimeScale.Type.First90D;
                case Buttons.Code.TimeLast90Days:
                    return TimeScale.Type.Last90D;
                case Buttons.Code.TimeFirst30Days:
                    return TimeScale.Type.First30D;
                case Buttons.Code.TimeFirst7Days:
                    return TimeScale.Type.First7D;
                case Buttons.Code.TimeLast30Days:
                    return TimeScale.Type.Last30D;
                case Buttons.Code.TimeLast7Days:
                    return TimeScale.Type.Last7D;
                case Buttons.Code.TimeMonth:
                    return TimeScale.Type.Month;
                case Buttons.Code.TimeQuarter:
                    return TimeScale.Type.Quarter;
                case Buttons.Code.TimeWeek:
                    return TimeScale.Type.Week;
                case Buttons.Code.TimeYear:
                    return TimeScale.Type.Year;
                case Buttons.Code.TimeCustom:
                    return TimeScale.Type.Custom;
            }
            throw new NotImplementedException();
        }

        public static bool ChangeTo(Buttons.Code code)
        {
            Type newtype = ButtonCodeToType(code);
            if (type == newtype) { return false; }

            type = newtype;

            PeriodIndex = null;

            switch (type)
            {
                case Type.Week:
                    PeriodIndex = 0;
                    break;
                case Type.Month:
                    PeriodIndex = 1;
                    break;
                case Type.Quarter:
                    PeriodIndex = 2;
                    break;
                case Type.Year:
                    PeriodIndex = 3;
                    break;
            }
            if (PeriodIndex != null)
            {
                CurrentPeriod[(int)PeriodIndex] = LoadSales.WeekMonthQuarterYear_ValuesOfDates[LoadSales.AllDates.Count - 1][(int)PeriodIndex];
            }
            return true;
        }

        static bool? SearchTimeScaleSens = null;
        static bool[] PrevNextPossible = new bool[] { false, false };

        public static bool ChangePeriod(Buttons.Code code)
        {
            switch (code)
            {
                case Buttons.Code.TimePeriodLast:
                    // is there any data later than current?
                    for (int i = dateIndexMax + 1; i < LoadSales.AllDates.Count; i++)
                    {
                        if (LoadSales.AllDates[i].ContainsAnySelectedGame)
                        {
                            // yes
                            CurrentPeriod[(int)PeriodIndex] = LoadSales.WeekMonthQuarterYear_ValuesOfDates[LoadSales.AllDates.Count - 1][(int)PeriodIndex];
                            SearchTimeScaleSens = false;
                            return true;
                        }
                    }
                    return false;
                case Buttons.Code.TimePeriodFirst:
                    // is there any data earlier than current?
                    for (int i = dateIndexMin - 1; i >= 0; i--)
                    {
                        if (LoadSales.AllDates[i].ContainsAnySelectedGame)
                        {
                            // yes
                            CurrentPeriod[(int)PeriodIndex] = LoadSales.WeekMonthQuarterYear_ValuesOfDates[0][(int)PeriodIndex];
                            SearchTimeScaleSens = true;
                            return true;
                        }
                    }
                    return false;
                case Buttons.Code.TimePeriodNext:
                    return PeriodPlusOne(+1);
                case Buttons.Code.TimePeriodPrevious:
                    return PeriodPlusOne(-1);
                default:
                    throw new NotImplementedException();
            }
        }

        static bool PeriodPlusOne(int sens)
        {
            int total = LoadSales.WeekMonthQuarterYear_ValuesOfDates[LoadSales.AllDates.Count-1][(int)PeriodIndex] + 1;
            CurrentPeriod[(int)PeriodIndex] =
                (CurrentPeriod[(int)PeriodIndex]
                + sens
                + total) % total;

            SearchTimeScaleSens = sens > 0;

//#warning TODO check if has really changed (return bool)
            return true;
        }

        public static bool IsSelected(int year, int month, int day)
        {
            int dateIndex = FindDateIndex(year, month, day);
            switch (type)
            {
                case Type.Week:
                    return LoadSales.WeekMonthQuarterYear_ValuesOfDates[dateIndex][0] == CurrentPeriod[0];
                case Type.Month:
                    return LoadSales.WeekMonthQuarterYear_ValuesOfDates[dateIndex][1] == CurrentPeriod[1];
                case Type.Quarter:
                    return LoadSales.WeekMonthQuarterYear_ValuesOfDates[dateIndex][2] == CurrentPeriod[2];
                case Type.Year:
                    return LoadSales.WeekMonthQuarterYear_ValuesOfDates[dateIndex][3] == CurrentPeriod[3];
            }

            return dateIndex >= dateIndexMin && dateIndex <= dateIndexMax;
        }

        static int dateIndexMin = -1, dateIndexMax = -1;
        static int[] CurrentPeriod = new int[] { -1, -1, -1, -1 };
        static int? PeriodIndex = null;

        public static void FindSuitableTimeFrame()
        {

            /*
             * find DateIndex Min and Max
             * if no date of period ContainsAnyGameSelected
             * then choose another period
             */

            switch (type)
            {
                case Type.All:
                    dateIndexMin = EarliestDateAfter(0);
                    dateIndexMax = LatestDateBefore(LoadSales.AllDates.Count - 1);
                    HideChangePeriodButtons();
                    return;

                case Type.First90D:
                    dateIndexMin = EarliestDateAfter(0);
                    dateIndexMax = LatestDateBefore(dateIndexMin + 89);
                    HideChangePeriodButtons();
                    return;
                case Type.Last90D:
                    dateIndexMax = LatestDateBefore(LoadSales.AllDates.Count - 1);
                    dateIndexMin = EarliestDateAfter(dateIndexMax - 89);
                    HideChangePeriodButtons();
                    return;

                case Type.First7D:
                    dateIndexMin = EarliestDateAfter(0);
                    dateIndexMax = LatestDateBefore(dateIndexMin + 6);
                    HideChangePeriodButtons();
                    return;
                case Type.First30D:
                    dateIndexMin = EarliestDateAfter(0);
                    dateIndexMax = LatestDateBefore(dateIndexMin + 29);
                    HideChangePeriodButtons();
                    return;
                case Type.Last30D:
                    dateIndexMax = LatestDateBefore(LoadSales.AllDates.Count - 1);
                    dateIndexMin = EarliestDateAfter(dateIndexMax - 29);
                    HideChangePeriodButtons();
                    return;
                case Type.Last7D:
                    dateIndexMax = LatestDateBefore(LoadSales.AllDates.Count - 1);
                    dateIndexMin = EarliestDateAfter(dateIndexMax - 6);
                    HideChangePeriodButtons();
                    return;
            }

            // type == Period : Year or Quarter or Month or Week
            dateIndexMin = -1;
            do
            {
                // any sales available in period ?
                for (int i = 0; i < LoadSales.AllDates.Count; i++)
                {
                    if (
                        LoadSales.WeekMonthQuarterYear_ValuesOfDates[i][(int)PeriodIndex] == CurrentPeriod[(int)PeriodIndex]
                        && LoadSales.AllDates[i].ContainsAnySelectedGame)
                    {
                        if (dateIndexMin < 0)
                        {
                            dateIndexMin = i;
                        }
                        dateIndexMax = i;
                    }
                }
                if (dateIndexMin < 0)
                {
                    // nothing found
                    if (SearchTimeScaleSens == null)
                    {
                        SearchTimeScaleSens = true;
                    }
                    ChangePeriod((bool)SearchTimeScaleSens 
                        ? Buttons.Code.TimePeriodNext 
                        : Buttons.Code.TimePeriodPrevious);
                }
            } while (dateIndexMin < 0);
            SearchTimeScaleSens = null;

            foreach (Button<Texture2D> b in ChangeTimePeriodMinusButtons)
            {
                Buttons.ShowHideButton(b, AnyDataBeforeAfterPeriod(false));
            }
            foreach (Button<Texture2D> b in ChangeTimePeriodPlusButtons)
            {
                Buttons.ShowHideButton(b, AnyDataBeforeAfterPeriod(true));
            }
        }

        static void HideChangePeriodButtons()
        {
            foreach (Button<Texture2D> b in ChangeTimePeriodPlusButtons)
            {
                Buttons.ShowHideButton(b, false);
            }
            foreach (Button<Texture2D> b in ChangeTimePeriodMinusButtons)
            {
                Buttons.ShowHideButton(b, false);
            }
        }


        public static bool AnyDataBeforeAfterPeriod(bool sens)
        {
            int
                start = sens ? (dateIndexMax + 1) : (dateIndexMin - 1),
                end = sens ? (LoadSales.AllDates.Count - 1) : 0;
            for (int i = start; sens ? (i <= end) : (i >= end); i += (sens ? 1 : -1))
            {
                if (LoadSales.AllDates[i].ContainsAnySelectedGame) { return true; }
            }
            return false;
        }

        static int EarliestDateAfter(int start)
        {
            for (int i = Math.Max(0, start); i < LoadSales.AllDates.Count - 1; i++)
            {
                if (LoadSales.AllDates[i].ContainsAnySelectedGame) { return i; }
            }
            throw new Exception();
        }
        static int LatestDateBefore(int end)
        {
            for (int i = Math.Min(LoadSales.AllDates.Count - 1, end); i >= 0; i--)
            {
                if (LoadSales.AllDates[i].ContainsAnySelectedGame) { return i; }
            }
            throw new Exception();
        }


        static int FindDateIndex(int year, int month, int day)
        {
//#warning TODO can be faster by caching previous found index
            LoadSales.DateWithGames thisDate = new LoadSales.DateWithGames(year, month, day);
            for (int i = 0; i < LoadSales.AllDates.Count; i++)
            {
                if (LoadSales.AllDates[i].Equals(thisDate))
                {
                    return i;
                }
            }
            throw new Exception();
        }
        public static void Draw()
        {
            MaximinusGame.DrawStringOutlined(
                Totals.AllDatesForTimeScale.Count.ToString("0") +
                " days between " +
                LoadSales.AllDates[dateIndexMin].ToString() +
                " and " +
                LoadSales.AllDates[dateIndexMax].ToString(),
                MaximinusGame.Instance.Font,
                new Vector2(BigGraph.Left + BigGraph.Width * 3 / 4, BigGraph.Bottom + 20 + Buttons.Height * 1 / 6),
                Color.White, 1, 1);
            MaximinusGame.DrawStringOutlined(
                PeriodName,
                MaximinusGame.Instance.Font,
                new Vector2(BigGraph.Left + BigGraph.Width * 3 / 4, BigGraph.Bottom + 20 + Buttons.Height * 7/6),
                Color.White, 1, 1);
        }
        static String PeriodName
        {
            get
            {
                LoadSales.DateWithGames startDate = LoadSales.AllDates[dateIndexMin];
                switch (type)
                {
                    case Type.Week:
                        return "Week #" + (1 + LoadSales.WeekMonthQuarterYear_ValuesOfDates[dateIndexMin][0]);
                    case Type.Month:
                        return LoadSales.DateWithGames.MonthString(startDate.Month - 1) + " " + startDate.Year;
                    case Type.Quarter:
                        return "Q" + (((startDate.Month - 1) / 3) + 1) + " " + startDate.Year;
                    case Type.Year:
                        return "Fiscal Year " + startDate.FiscalYear;
                }
                return "";
            }
        }
    }
}
