﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace SalesVizu
{
    public class Totals
    {
        public static int Top { get { return Value.Top; } }

        public static int[] MaxCount = new int[3];

        class TotalLine
        {
            public enum Type { Labels, Value, Sales, Trials, Percentage };


            public float Total;
            public float AvgDay;
            readonly Rectangle
                RectTotal,
                RectAvgDay;
            readonly String Label;
            readonly Type type;

            public int Top { get { return RectTotal.Top; } }


            public TotalLine(Type type, String Label, int index)
            {
                this.type = type;
                this.Label = Label;

                const int gapY = 25;

                //const int gapY = 50;
                //const int gapX = 200;

                //RectLabel = new Rectangle(
                //    20,// + index * X,
                //    850 + index * Y,
                //    100,
                //    Y);
                RectTotal = new Rectangle(
                    //gapX * 5/4 + index * gapX,
                    //1000 - 300 - 50,
                    20,
                    875 + index * gapY,
                    100,
                    gapY);
                RectAvgDay = new Rectangle(
                    //gapX * 5 / 4 + index * gapX,
                    //1000 - 300 - 20,
                    150,
                    875 + index * gapY,
                    100,
                    gapY);
            }
            public void Reset()
            {
                Total = 0;
                AvgDay = 0;
            }
            public void ComputeAverage()
            {
                AvgDay = Total / AllDatesForTimeScale.Count;
            }
            public void Draw()
            {
                //MaximinusGame.Draw2D.DrawStringOutlinedNotCentered(
                //    Label,
                //    new Vector2(RectLabel.X, RectLabel.Y - RectLabel.Height/2),
                //    Color.White, 1, 1);
                MaximinusGame.DrawStringOutlinedNotCentered( //.DrawStringOutlined(
                    (type == Type.Labels ? "Total" : (Total.ToString(Format(0)) + Symbol(0))),
                    //MaximinusGame.Draw2D.Font,
                    new Vector2(RectTotal.X /*+ (type == Type.Percentage ? 60 : 0)*/, RectTotal.Y),
                    type == Type.Value ? Color.Gold : Color.White, 1, 1);
                if (type != Type.Percentage)
                {
                    MaximinusGame.DrawStringOutlinedNotCentered( //.DrawStringOutlined(
                        (type == Type.Labels ? "Average" : (AvgDay.ToString(Format(1)) + Symbol(1))),
                        //MaximinusGame.Draw2D.Font,
                        new Vector2(RectAvgDay.X, RectAvgDay.Y),
                        Color.White, 1, 1);
                }
            }
            String Format(int column)
            {
                switch (type)
                {
                    case Type.Sales:
                    case Type.Trials:
                        return "0";
                    case Type.Value:
                        return "0";
                    case Type.Percentage:
                        return "0.00";
                }
                throw new Exception();
            }
            String Symbol(int column)
            {
                String ret = "";
                switch (type)
                {
                    case Type.Value: ret = (column == 1 ? " " + Currency.Symbol + "/Day" : " " + Currency.Symbol); break;
                    case Type.Percentage: ret = " % Conversion Rate"; break;
                    case Type.Sales: ret = (column == 1 ? " S/Day" : " Sales"); break;
                    case Type.Trials: ret = (column == 1 ? " T/Day" : " Trials"); break;
                }
                return ret;
            }
        }

        static TotalLine
            Labels = new TotalLine(TotalLine.Type.Labels, "", -2),
            Value = new TotalLine(TotalLine.Type.Value, "Value", -1),
            SalesUnits = new TotalLine(TotalLine.Type.Sales, "Sales", 0),
            TrialsUnits = new TotalLine(TotalLine.Type.Trials, "Trials", 1),
            ConvRate = new TotalLine(TotalLine.Type.Percentage, "Conv Rate", 3);
        //public static int DayCount = -1;

        public static List<float> ValueMSPbyGame = new List<float>(), ValueMSPbyCountry = new List<float>(), ConvRateByGame = new List<float>();

        public static int DataChangeFrame = -1;

        public static List<TimeFrame.Date> AllDatesForTimeScale = new List<TimeFrame.Date>();

        public static void Rebuild()
        {

            Value.Reset();
            SalesUnits.Reset();
            TrialsUnits.Reset();
            ConvRate.Reset();

            AllDatesForTimeScale.Clear();

            ValueMSPbyGame.Clear();
            ValueMSPbyCountry.Clear();
            ConvRateByGame.Clear();

            TimeScale.FindSuitableTimeFrame();

            for (int m = 0; m < MaxCount.Length; m++)
            {
                MaxCount[m] = 0;
            }

            for (int c = 0; c < ByCountry.NbCountries; c++)
            {
                ValueMSPbyCountry.Add(0);
            }

            for (int g = 0; g < Data.SalesByGame.Length; g++)
            {
                ValueMSPbyGame.Add(0);
                Point convRateData = Point.Zero;

                if (Games.Selected[g])
                {
                    List<DailySales> thisGameSales = Data.SalesByGame[g];

                    foreach (DailySales s in thisGameSales)
                    {
                        if (TimeScale.IsSelected(s.Year, s.Month, s.Day))
                        {

                            Value.Total += s.ValueMSP * Currency.MSP2Currency;
                            SalesUnits.Total += s.Sales;
                            TrialsUnits.Total += s.Trials;

                            TimeFrame.Date thisDate = new TimeFrame.Date();
                            thisDate.SalesTrialsValuesByGame = new int[3, Data.Games.Length];
                            thisDate.Year = s.Year;
                            thisDate.Month = s.Month;
                            thisDate.Day = s.Day;

                            int insertIndex = InsertIfNotPresent(thisDate);

                            int salesAndUnsoldTrials = Math.Max(s.Trials, s.Sales);

                            AllDatesForTimeScale[insertIndex].SalesTrialsValuesByGame[(int)BigGraph.Type.Sales, g] += s.Sales;
                            AllDatesForTimeScale[insertIndex].SalesTrialsValuesByGame[(int)BigGraph.Type.SalesAndUnsoldTrials, g] += salesAndUnsoldTrials;
                            AllDatesForTimeScale[insertIndex].SalesTrialsValuesByGame[(int)BigGraph.Type.ValueMSP, g] += s.ValueMSP;

                            MaxCount[(int)BigGraph.Type.Sales] = Math.Max(
                                MaxCount[(int)BigGraph.Type.Sales],
                                AllDatesForTimeScale[insertIndex].Max((int)BigGraph.Type.Sales));
                            MaxCount[(int)BigGraph.Type.SalesAndUnsoldTrials] = Math.Max(
                                MaxCount[(int)BigGraph.Type.SalesAndUnsoldTrials],
                                AllDatesForTimeScale[insertIndex].Max((int)BigGraph.Type.SalesAndUnsoldTrials));
                            MaxCount[(int)BigGraph.Type.ValueMSP] = Math.Max(
                                MaxCount[(int)BigGraph.Type.ValueMSP],
                                AllDatesForTimeScale[insertIndex].Max((int)BigGraph.Type.ValueMSP));




                            //if (AllDates.Contains(thisDate) == false)
                            //{
                            //    AllDates.Add(thisDate);
                            //}

                            ValueMSPbyGame[ValueMSPbyGame.Count - 1] += s.ValueMSP;

                            convRateData.X += s.Sales;
                            convRateData.Y += s.Trials;

                            for (int c = 0; c < ByCountry.NbCountries; c++)
                            {
                                ValueMSPbyCountry[c] += s.ValueMSPbyCountry[c];
                            }
                        }
                    }
                }
                ConvRateByGame.Add((float)convRateData.X / Math.Max(1, convRateData.Y));
            }

            for (int g = 0; g < ValueMSPbyGame.Count; g++)
            {
                if (Value.Total > 0)
                {
                    ValueMSPbyGame[g] /= Value.Total / Currency.MSP2Currency;
                }
                else
                {
                    ValueMSPbyGame[g] = 0;
                }
            }
            for (int c = 0; c < ValueMSPbyCountry.Count; c++)
            {
                if (Value.Total > 0)
                {
                    ValueMSPbyCountry[c] /= Value.Total / Currency.MSP2Currency;
                }
                else
                {
                    ValueMSPbyCountry[c] = 0;
                }
            }

            Value.ComputeAverage();
            TrialsUnits.ComputeAverage();
            SalesUnits.ComputeAverage();
            ConvRate.Total = SalesUnits.Total / TrialsUnits.Total * 100;
            
            DataChangeFrame = MaximinusGame.CurrentFrame;
        }

        static int InsertIfNotPresent(TimeFrame.Date thisDate)
        {
            if (AllDatesForTimeScale.Count == 0)
            {
                AllDatesForTimeScale.Add(thisDate);
                return 0;
            }


            for (int i = 0; i < AllDatesForTimeScale.Count; i++)
            {
                if(thisDate.Equals(AllDatesForTimeScale[i]))
                {
                    return i;
                }
                if (thisDate.IsEarlierThan(AllDatesForTimeScale[i]))
                {
                    AllDatesForTimeScale.Insert(i, thisDate);                    
                    return i;
                }
            }

            AllDatesForTimeScale.Add(thisDate);
            return AllDatesForTimeScale.Count - 1;
        }

        public static void Draw(bool anyGameSelected)
        {
            Labels.Draw();
            if (anyGameSelected)// && LoadSales.Rebuilding == false)
            {
                Value.Draw();
                SalesUnits.Draw();
                TrialsUnits.Draw();
                ConvRate.Draw();
            }
        }    
    }
}
